//
// dep-test.c
//
// This program tests DEP by jumping to the code segment, data segment, heap
// and stack and reporting whether an access violation exception is generated.
// It is compiled twice, with the /NXcompat linker option and without.

#include <windows.h>

#define _WIN32_WINNT 0x0500
#include <winternl.h>

#include <string.h>
#include <malloc.h>

//
// Returns the address of an exported function in a DLL
//

FARPROC resolve_func(char* dll, char* func)
{
    HMODULE module;
    FARPROC addr;
    
    module = LoadLibrary(dll);
    if (module == NULL) {
        printf("Failed to load DLL %s\n", dll);
        exit(1);
    }

    addr = GetProcAddress(module, func);
    if (addr == NULL) {
        printf("Failed to resolve function %s in DLL %s\n", func, dll);
        exit(1);
    }

    return addr;
}

const DWORD ProcessExecuteFlags = 0x22;

typedef struct {
    unsigned int ExecuteDisable : 1;
    unsigned int ExecuteEnable : 1;
    unsigned int DisableThunkEmulation : 1;
    unsigned int Permanent : 1;
    unsigned int ExecuteDispatchEnable : 1;
    unsigned int ImageDispatchEnable : 1;
    unsigned int DisableExceptionChainValidation : 1;
    unsigned int Spare : 25;
} KEXECUTE_OPTIONS;


//
// Returns the process execution flags
//

KEXECUTE_OPTIONS get_process_flags()
{
    FARPROC NtQueryInformationProcess;
    NTSTATUS status;
    KEXECUTE_OPTIONS flags;
    DWORD len;

    NtQueryInformationProcess = resolve_func("ntdll.dll",
        "NtQueryInformationProcess");

    status = NtQueryInformationProcess(GetCurrentProcess(),
                                       ProcessExecuteFlags,
                                       &flags,
                                       sizeof(flags),
                                       &len);

    if (status < 0) {
        printf("NtQueryInformationProcess returned error code 0x%x\n", status);
        exit(1);
    }

    if (len != 4) {
        printf("NtQueryInformationProcess returned unexpected length %d\n",
               len);
        exit(1);
    }

    return flags;
}


void dump_process_flags(KEXECUTE_OPTIONS flags)
{
    printf("Process execution flags   : 0x%x\n"
        "    ExecuteDisable        : %d\n"
        "    ExecuteEnable         : %d\n"
        "    DisableThunkEmulation : %d\n"
        "    Permanent             : %d\n"
        "    ExecuteDispatchEnable : %d\n"
        "    ImageDispatchEnable   : %d\n"
        "    DisableExceptionChainValidation : %d\n",
        *(DWORD*)&flags,
        flags.ExecuteDisable,
        flags.ExecuteEnable,
        flags.DisableThunkEmulation,
        flags.Permanent,
        flags.ExecuteDispatchEnable,
        flags.ImageDispatchEnable,
        flags.DisableExceptionChainValidation);
}


void dep_test(FARPROC addr)
{
    __try {
        addr();
        printf("ok\n");
    } __except(EXCEPTION_EXECUTE_HANDLER) {
        printf("access violation\n");
    }
}

void code_text() {
    return;
}

char code_data[4096];

void main()
{
    char code_stack[4096];
    char* code_heap = (char*)malloc(4096);

    KEXECUTE_OPTIONS flags = get_process_flags();

    dump_process_flags(flags);

    printf("\nRunning tests:\n");

    printf("    text  : ");

    dep_test((FARPROC)code_text);

    printf("    data  : ");

    memcpy(code_data, code_text, 4096);
    dep_test((FARPROC)code_data);

    printf("    heap  : ");

    memcpy(code_heap, code_text, 4096);
    dep_test((FARPROC)code_heap);

    printf("    stack : ");

    memcpy(code_stack, code_text, 4096);
    dep_test((FARPROC)code_stack);
}
