//
// gs-perf.c
//
// This program measures the worst case slowdown introduced by the /GS compiler
// option. It executes 100 million iterations of a function that returns the
// address of a local variable and measures how long it takes with and without
// GS.


#include <windows.h>
#include <stdio.h>

void* func_nogs();
void* func_gs();

// 100 million iterations take about 1.5 seconds on a 2.4GHz
// Intel Core2 Duo, adjust as necessary for your CPU

#define ITERATIONS 100000000

int main()
{
    DWORD start, elapsed_nogs, elapsed_gs;
    DWORD i;

    printf("Testing worst case GS performance:\n\n");

    // Without GS

    start = GetTickCount();

    for (i = 0; i < ITERATIONS; i++)
        func_nogs();

    elapsed_nogs = GetTickCount() - start;

    printf("GS disabled: %u calls per tick\n", ITERATIONS/elapsed_nogs);

    // With GS

    start = GetTickCount();

    for (i = 0; i < ITERATIONS; i++)
        func_gs();

    elapsed_gs = GetTickCount() - start;

    printf("GS enabled : %u calls per tick\n", ITERATIONS/elapsed_gs);

    // Calculate the percentage slowdown

    printf("\nGS slowdown: %u%%\n", 100*(elapsed_gs-elapsed_nogs)/elapsed_gs);

    return 0;
}
